#version 430 core

// Input variables
in VertexData
{
	vec3 normal_ws;
	vec3 position_ws;
	vec2 tex_coord;
	vec3 tangent_ws;
	vec3 bitangent_ws;
} inData;

// Output variables
layout (location = 0) out vec4 final_color;

// Data of the camera
layout (std140, binding = 0) uniform CameraData
{
	mat4 projection;		// Projection matrix
	mat4 projection_inv;	// Inverse of the projection matrix
	mat4 view;				// View matrix
	mat4 view_inv;			// Inverse of the view matrix
	mat3 view_it;			// Inverse of the transpose of the top-left part 3x3 of the view matrix
	vec3 eye_position;		// Position of the eye in world space
};

// Data of the light
struct PhongLight
{
	// See the C++ code for the documentation to individual attributes
	vec4 position;
	vec3 ambient;
	vec3 diffuse;
	vec3 specular;
	vec3 spot_direction;
	float spot_exponent;
	float spot_cos_cutoff;
	float atten_constant;
	float atten_linear;
	float atten_quadratic;
};
layout (std140, binding = 1) uniform PhongLightsData
{
	vec3 global_ambient_color;
	int lights_count;
	PhongLight lights[8];
};

// Data of the material
uniform vec3 material_diffuse;
uniform vec3 material_fresnel;
uniform float material_shininess_tangent;
uniform float material_shininess_bitangent;

//-----------------------------------------------------------------------

void main()
{
	// Compute the lighting
	vec3 N = normalize(inData.normal_ws);
	vec3 T = normalize(inData.tangent_ws);
	vec3 B = normalize(inData.bitangent_ws);
	vec3 V = normalize(eye_position - inData.position_ws);		// This is 'Eye' variable in previous lectures

	// In this lecture, we ignore the global ambient light
	vec3 amb = vec3(0.0);
	vec3 dif = vec3(0.0);
	vec3 spe = vec3(0.0);

	for (int i = 0; i < lights_count; i++)
	{
		vec3 L_not_normalized = lights[i].position.xyz - inData.position_ws * lights[i].position.w;
		vec3 L = normalize(L_not_normalized);
		vec3 H = normalize(L + V);

		// Compute the ambient part. Ambient light is a bit tricky, because it does not exist in PBR.
		// We use the average of diffuse and fresnel and also very small ambient light, so that we can see
		// at least some of the shape of the object in the dark parts, do not make the light too bright.
		amb += (material_diffuse + material_fresnel)*0.5 * lights[i].ambient;

		// Task 4: Compute the diffuse part.
		//		Hint: Vectors N, T, B, V, H, and L are prepared, light's intensity and color is in lights[i].diffuse,
		//			Color_diff is in material_diffuse, F_0 is in material_fresnel
		dif += lights[i].diffuse * 0.0000000;

		// Task 4: Compute the specular part.
		//		Hint: Copy-paste function FresnelSchlick from CookTorrance_fragment.glsl
		//		Hint: Vectors N, T, B, V, H, and L are prepared, light's intensity and color is in lights[i].diffuse,
		//			F_0 is in material_fresnel, shin_T and shin_B are in material_shininess_tangent and material_shininess_bitangent
		spe += lights[i].diffuse * 0.0000000;
	}

	// Add all three parts of the color together
	vec3 final_light = amb + dif + spe;
	final_color = vec4(final_light, 1.0);
}

//-----------------------------------------------------------------------